#ifndef PRINTERPARSER_H
#define PRINTERPARSER_H

#include<vector>

#include "printermodel.h"

/**
 * @brief Class parsing printer definitions and creating model instances.
 *
 * Parses a file where a printer definition uses the following format
 * <queuename>Tab<server>Tab<location>Tab<driver>Tab<format>Tab<colourmode>Tab<duplex>Tab
 * <staple>Tab<punchunit>Tab<booklet>Tab<driver>Tab<lpoptions>Tab<responsible>
 *
 * Note: The expected encoding for the file is UTF-16 Little Endian
 *
 * <queuename> values: name of the queue on the server
 * <server> values: hostname of the server (with or without domain?)
 * <location> values: location field from the inventory
 * <driver> values: name of the PPD file with extension
 * <format> values: A4; A3
 * <colourmode> values: colour; grayscale
 * <duplex> values: duplex; simplex
 * <staple> values: staple; nostaple
 * <punchunit> values: punch; nopunch
 * <booklet> values: booklet; nobooklet
 * <scanner> values: A4scanner; A3scanner; noscanner
 * <lpoptions> values: complete string for the driver specific lpoptions including the -o parameter
 * <responsible> values: string with name and office of the person responsible for this printer
 *
 * Comments are marked by a "#" at the beginning of a line
 */
class PrinterParser {
public:
    /**
     * @brief PrinterParser constructor
     */
    PrinterParser();

    /**
     * @brief Parse the given printer definition file and return a list of printer models
     * @param file Path to definitions file
     * @return List of models as vector
     */
    std::vector<PrinterModel>* getPrinters(std::string file);

private:

    /**
     * @brief Creates a printer model from the given printer string
     * @param printerString printer definition as string
     * @return Printer model instance
     */
    PrinterModel createPrinterModel(std::string printerString);

};

struct PrinterParserException : public std::exception {
   const char * what () const throw () {
      return "Parser Exception";
   }
};


#endif // PRINTERPARSER_H
