#ifndef SMBUTILITY_H
#define SMBUTILITY_H

#include <exception>
#include <string>

#include "configutility.h"

class SMBException: public std::exception {
private:
    std::string message;
public:
    explicit SMBException(const std::string& message) {
        this->message = message;
    };
    const char* what() const noexcept override {
        return message.c_str();
    }
};

/**
 * @brief Class providing functions to copy files from a smb share
 */
class SMBUtility {

public:

    SMBUtility();

    /**
     * @brief Executes the given command and returns the output
     * @param cmd Command to execute
     * @return
     */
    std::string exec(const std::string cmd);

    /**
     * @brief Read the remote file and copy to local filesystem using the given credentials
     * @param user Username
     * @param password password
     * @param domain SMB Domain
     * @return Local file path
     */
    std::string getRemoteConfigFile(std::string user, std::string password, std::string domain);

    /**
     * @brief Read the remote file and copy to local filesystem without using credentials
     * @return Local file path
     */
    std::string getRemoteConfigFileAnonymous();

    /**
     * @brief Get the smb uri using the config.
     *
     * The printer definitions filename will not be included in the uri
     * @return Uri as string
     */
    std::string getSmbURI();

    /**
     * @brief Get the smb uri using the config
     * @param includeFile Set if printer definitions file should be included
     * @return Uri as string
     */
    std::string getSmbURI(bool includeFile);

    /**
     * @brief Get uri from given parameters
     * @param host Host
     * @param share Share
     * @param file File
     * @return Uri as string
     */
    std::string getSmbURI(std::string host, std::string share, std::string file);

    static void setDebugLevel(int debugLevel);

    /**
     * @brief Temp dir to copy remote files to
     */
    const std::string TMP_PRINTER_FILE = "/tmp/printers_config.txt";

private:

    ConfigUtility *cfg;

    const std::string CMD = "smbclient ";
    const std::string CMD_DOMAIN_PREFIX = " -W \"";
    const std::string CMD_DOMAIN_POSTFIX = "\" ";
    const std::string CMD_ANON = " --no-pass ";
    const std::string CMD_CREDENTIALS_PREFIX ="--user=\"";
    const std::string CMD_CREDENTIALS_POSTFIX = "\" ";
    const std::string CMD_SOURCE_PREFIX = " --command 'get \"";
    const std::string CMD_DEST_PREFIX = "\" \"";
    const std::string CMD_DEST_POSTFIX = "\"'";
    const std::string CMD_OUTPUT_POSTFIX = " 2>&1";
    const std::string CMD_NO_OUTPUT_POSTFIX = "> /dev/null 2>&1";

    static int debugLevel;

};

#endif // SMBUTILITY_H
