#include "tablemodel.h"

#include <iostream>
#include <regex>
#include <sstream>

#include <QtDebug>
#include <QColor>
#include <QBrush>

#include <QJSEngine>

#include "smbutility.h"

TableModel::TableModel(QObject *parent)
    : QAbstractTableModel(parent) {

    cfg = ConfigUtility::getInstance();
    cupsUtility = new CupsUtility();
    parser = new PrinterParser();
    smbUtility = new SMBUtility();
    tableData = new std::vector<PrinterModel>();
    visibleData = new std::vector<PrinterModel>();
    initHeaderData();
}

std::vector<int> TableModel::getColumnWidths() {
    return *columnWidths;
}

void TableModel::initHeaderData() {
    headerFields = new std::vector<std::string>();
    columnWidths = new std::vector<int>();
    headerFields->push_back("Name");
    columnWidths->push_back(150);
    headerFields->push_back("Server");
    columnWidths->push_back(150);
    headerFields->push_back("Location");
    columnWidths->push_back(150);
    headerFields->push_back("Driver");
    columnWidths->push_back(200);
    headerFields->push_back("Format");
    columnWidths->push_back(100);
    headerFields->push_back("Color");
    columnWidths->push_back(100);
    headerFields->push_back("Duplex");
    columnWidths->push_back(100);
    headerFields->push_back("Staple");
    columnWidths->push_back(100);
    headerFields->push_back("Punch");
    columnWidths->push_back(100);
    headerFields->push_back("Booklet");
    columnWidths->push_back(100);
    headerFields->push_back("Scanner");
    columnWidths->push_back(100);
    headerFields->push_back("Opts");
    columnWidths->push_back(200);
    headerFields->push_back("Comment");
    columnWidths->push_back(250);

    loadColumnWidths();
}

void TableModel::clearFilter() {
    visibleData = tableData;
    emit layoutChanged();
}

void TableModel::filter(QString filterString) {
    visibleData = new std::vector<PrinterModel>();

    std::regex regex (filterString.toStdString(), std::regex::icase);
    for (int j = 0; j < tableData->size(); j++) {
        PrinterModel m = tableData->at(j);
        if (m.matches(regex)) {
            visibleData->push_back(m);
        }
    }
    emit layoutChanged();
}

void TableModel::handleItemClicked(int row, int column) {
    this->selection = row;
    emit layoutChanged();
    emit layoutChanged();
}

void TableModel::initTableData(std::string path) {
    tableData = parser->getPrinters(path);
    visibleData = tableData;
    emit layoutChanged();
}

bool TableModel::needCredentialsForPrinterList() {
    try {
        smbUtility->getRemoteConfigFileAnonymous();
    } catch (SMBException*) {
        return true;
    }
    return false;
}

bool TableModel::readRemotePrinterListAnonymous() {
    try {
        std::string localPath = smbUtility->getRemoteConfigFileAnonymous();
        initTableData(localPath);
        return true;
    } catch (SMBException*) {
        return false;
    }
    catch (PrinterParserException*) {
        return false;
    }
}

bool TableModel::readRemotePrinterListAuthenticated(QString user, QString password, QString domain) {
    try {
        qDebug() << "Loading remote printer file...";
        std::string localPath = smbUtility->getRemoteConfigFile(user.toStdString(), password.toStdString(), domain.toStdString());
        initTableData(localPath);
        return true;
    } catch (SMBException* e) {
        qDebug() << "Loading remote list failed: " << e->what();
        qjsEngine(this) -> throwError(e->what());
        return false;
    }
    catch (PrinterParserException*) {
        return false;
    }
}

bool TableModel::loadLocalPrinterDefinitions() {
    try {
        std::vector<PrinterModel> *localPrinters = this->parser->getPrinters(smbUtility->TMP_PRINTER_FILE);
        if (localPrinters->size() > 0) {
            tableData = localPrinters;
            visibleData = tableData;
            emit layoutChanged();
            return true;
        } else {
            return false;
        }
    } catch (PrinterParserException*) {
        std::cout << "Parser Exception" << std::endl;
        return false;
    }
}

void TableModel::storeColumnWidth(int column, int width) {
    columnWidths->at(column) = width;
    storeColumnWidths();
}

int TableModel::rowCount(const QModelIndex&) const {
    return visibleData->size();
}

int TableModel::columnCount(const QModelIndex&) const {
    return headerFields->size();
}

bool TableModel::isCellSelected(int row, int column) {
    return row == this->selection;

}

int TableModel::getColumnWidth(int column) {
    int width;
    try {
        width = columnWidths->at(column);
    } catch (std::out_of_range) {
        width = 50;
    }
    return width;
}

Q_INVOKABLE QVariant TableModel::headerData(int section, Qt::Orientation orientation, int role) const {
    if(role == Qt::DisplayRole){
        if(orientation == Qt::Horizontal)
            return QString(headerFields->at(section).c_str());
    }
    return QVariant();
}

QVariant TableModel::data(const QModelIndex &index, int role) const {
    switch (role) {
    case Qt::DisplayRole: {
        PrinterModel printer = visibleData->at(index.row());
        std::string fieldValue = printer.getAtIndexAsStr(index.column());
        return QString("%1").arg(fieldValue.c_str());
    }

    default:
        break;
    }
    return QVariant();
}

QHash<int, QByteArray> TableModel::roleNames() const {
    return { {Qt::DisplayRole, "display"} };
}

PrinterModel TableModel::getSelectedPrinter() {
    return this->tableData->at(this->selection);
}

void TableModel::loadColumnWidths() {
    std::string columns;
    try {
        columns = cfg->getCfgOpt(cfg->CFG_NAME_UI_COLUMN_WIDTHS);
    } catch (ConfigException*) {
        //If columns were not set yet, store current values
        storeColumnWidths();
        return;
    }

    if (columns.length() == 0) {
        return;
    }
    std::stringstream ss(columns);
    int index = 0;

    for (int column; ss >> column;) {
        columnWidths->at(index) = column;
        if (ss.peek() == ',') {
            ss.ignore();
        }
        index++;
    }
}

void TableModel::storeColumnWidths() {
    std::ostringstream columns;
    std::copy(columnWidths->begin(), columnWidths->end(), std::ostream_iterator<int>(columns, ","));
    cfg->setCfgOPt(cfg->CFG_NAME_UI_COLUMN_WIDTHS, columns.str());
    cfg->saveUserCfg();
}


QString TableModel::addSelectedPrinter(QString driver) {
    if (this->selection == -1) {
        return QString("No printer selected");
    }
    PrinterModel selected = getSelectedPrinter();
    bool result = cupsUtility->addSmbPrinter(selected, driver.toStdString());
    if (result) {
        return QString("Printer added successfully");
    } else {
        return QString("Could not add printer");
    }
}

QVariant TableModel::getSelectedQueueName() {
    if (this->selection == -1) {
        return "n/a";
    }
    PrinterModel selected = getSelectedPrinter();
    return QString(selected.getName().c_str());
}

Q_INVOKABLE QVariant TableModel::getSelectedServer() {
    if (this->selection == -1) {
        return "n/a";
    }
    PrinterModel selected = getSelectedPrinter();
    return QString(selected.getServer().c_str());
}

Q_INVOKABLE QVariant TableModel::getSelectedLocation() {
    if (this->selection == -1) {
        return "n/a";
    }
    PrinterModel selected = getSelectedPrinter();
    return QString(selected.getLocation().c_str());
}

Q_INVOKABLE QVariant TableModel::getSelectedDriver() {
    if (this->selection == -1) {
        return "n/a";
    }
    PrinterModel selected = getSelectedPrinter();
    return QString(selected.getDriver().c_str());
}

Q_INVOKABLE QVariant TableModel::getSelectedFormat() {
    if (this->selection == -1) {
        return "n/a";
    }
    PrinterModel selected = getSelectedPrinter();
    return QString(selected.getFormat().c_str());
}

Q_INVOKABLE QVariant TableModel::getSelectedColor() {
    if (this->selection == -1) {
        return "n/a";
    }
    PrinterModel selected = getSelectedPrinter();
    QString colorString = selected.getColor()? QString("Color"): QString("Grayscale");
    return colorString;
}

Q_INVOKABLE QVariant TableModel::getSelectedDuplex() {
    if (this->selection == -1) {
        return "n/a";
    }
    PrinterModel selected = getSelectedPrinter();
    QString duplexString = selected.getDuplex()? QString("Duplex"): QString("No Duplex");
    return duplexString;
}

Q_INVOKABLE QVariant TableModel::getSelectedStaple() {
    if (this->selection == -1) {
        return "n/a";
    }
    PrinterModel selected = getSelectedPrinter();
    QString duplexString = selected.getDuplex()? QString("Duplex"): QString("No Duplex");
    return duplexString;
}

Q_INVOKABLE QVariant TableModel::getSelectedPunch() {
    if (this->selection == -1) {
        return "n/a";
    }
    PrinterModel selected = getSelectedPrinter();
    QString punchString = selected.getPunch()? QString("Punch"): QString("No Punch");
    return punchString;
}

Q_INVOKABLE QVariant TableModel::getSelectedBooklet() {
    if (this->selection == -1) {
        return "n/a";
    }
    PrinterModel selected = getSelectedPrinter();
    QString bookletString = selected.getBooklet()? QString("Booklet"): QString("No Booklet");
    return bookletString;
}

Q_INVOKABLE QVariant TableModel::getSelectedScanner() {
    if (this->selection == -1) {
        return "n/a";
    }
    PrinterModel selected = getSelectedPrinter();
    return QString(selected.getScanner().c_str());
}

Q_INVOKABLE QVariant TableModel::getSelectedOpts() {
    if (this->selection == -1) {
        return "n/a";
    }
    PrinterModel selected = getSelectedPrinter();
    return QString(selected.getOpts().c_str());
}

Q_INVOKABLE QVariant TableModel::getSelectedComment() {
    if (this->selection == -1) {
        return "n/a";
    }
    PrinterModel selected = getSelectedPrinter();
    return QString(selected.getComment().c_str());
}

Q_INVOKABLE QVariant TableModel::getSelectedInstallCmd() {
    if (this->selection == -1) {
        return "No printer selected";
    }
    PrinterModel selected = getSelectedPrinter();
    return QString(cupsUtility->getCmd(selected).c_str());
}

QVariant TableModel::getSelectedInstallCmd(QString printer) {
    if (this->selection == -1) {
        return "No printer selected";
    }
    PrinterModel selected = getSelectedPrinter();
    return QString(cupsUtility->getCmd(selected,  printer.toStdString()).c_str());
}
