#ifndef TABLEMODEL_H
#define TABLEMODEL_H

#include <QGuiApplication>
#include <QQmlApplicationEngine>
#include <QQmlEngine>
#include <QAbstractTableModel>

#include <QMouseEvent>

#include <string>

#include "cupsutility.h"
#include "printerparser.h"
#include "printermodel.h"
#include "smbutility.h"

class TableModel : public QAbstractTableModel {
    Q_OBJECT

private:
    /**
     * @brief Init header fields and default column widths
     */
    void initHeaderData();

    void initTableData(std::string path = "");

    std::vector<int> *columnWidths;
    std::vector<std::string> *headerFields;
    std::vector<PrinterModel> *tableData;
    std::vector<PrinterModel> *visibleData;

    ConfigUtility *cfg;
    CupsUtility *cupsUtility;
    PrinterParser *parser;
    SMBUtility *smbUtility;

    int selection = -1;

    PrinterModel getSelectedPrinter();
    void loadColumnWidths();
    void storeColumnWidths();

public:

    /**
     * @brief Add the currently selected printer using the cups util
     * @param driver Optional driver
     * @return Result string
     */
    Q_INVOKABLE QString addSelectedPrinter(QString driver);

    /**
     * @brief Clear the table filter
     */
    Q_INVOKABLE void clearFilter();

    /**
     * @brief Apply a non case sensitive filter to the table data using the given string
     * @param filterString Filter string
     */
    Q_INVOKABLE void filter(QString filterString);

    /**
     * @brief Get the background color of a table cell by row and column index
     * @param row Cell row index
     * @param column Cell column index
     * @return Color as QVariant
     */
    Q_INVOKABLE bool isCellSelected(int row, int column);

    /**
     * @brief Get the width of the given column
     * @param column Column to get width of
     * @return Width as int
     */
    Q_INVOKABLE int getColumnWidth(int column);

    /**
     * @brief Handle clicks on a table cell
     * @param row Clicked row
     * @param column Clicked column
     */
    Q_INVOKABLE void handleItemClicked(int row, int column);

    /**
     * @brief Check if the smb host requires authentication
     * @return True if authentication is required, else false
     */
    Q_INVOKABLE bool needCredentialsForPrinterList();

    /**
     * @brief Read the printer definition list without authentication
     * @return True if successfully read, else false
     */
    Q_INVOKABLE bool readRemotePrinterListAnonymous();

    /**
     * @brief Read the printer definition with given credentials
     *
     * In case of failure, the functions throws an exception using the qt js engine.
     * @param user Username
     * @param password password
     * @return True if successfully read, else false
     */
    Q_INVOKABLE bool readRemotePrinterListAuthenticated(QString user, QString password, QString domain);

    /**
     * @brief Load local printer definitions
     * @return True if loaded successfully, else false
     */
    Q_INVOKABLE bool loadLocalPrinterDefinitions();

    /**
     * @brief Store the width of a column
     * @param column Column index
     * @param width Width
     */
    Q_INVOKABLE void storeColumnWidth(int column, int width);

    /**
     * @brief Constructor
     * @param parent Parent object
     */
    explicit TableModel(QObject *parent = nullptr);

    /**
     * @brief Get table column widths as vector
     * @return  Columns widths as vector
     */
    Q_INVOKABLE std::vector<int> getColumnWidths();

    Q_INVOKABLE QVariant getSelectedQueueName();
    Q_INVOKABLE QVariant getSelectedServer();
    Q_INVOKABLE QVariant getSelectedLocation();
    Q_INVOKABLE QVariant getSelectedDriver();
    Q_INVOKABLE QVariant getSelectedFormat();
    Q_INVOKABLE QVariant getSelectedColor();
    Q_INVOKABLE QVariant getSelectedDuplex();
    Q_INVOKABLE QVariant getSelectedStaple();
    Q_INVOKABLE QVariant getSelectedPunch();
    Q_INVOKABLE QVariant getSelectedBooklet();
    Q_INVOKABLE QVariant getSelectedScanner();
    Q_INVOKABLE QVariant getSelectedOpts();
    Q_INVOKABLE QVariant getSelectedComment();
    Q_INVOKABLE QVariant getSelectedInstallCmd();
    Q_INVOKABLE QVariant getSelectedInstallCmd(QString printer);

    Q_INVOKABLE QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const override;

    int rowCount(const QModelIndex & = QModelIndex()) const override;

    int columnCount(const QModelIndex & = QModelIndex()) const override;

    QVariant data(const QModelIndex &index, int role) const override;

    QHash<int, QByteArray> roleNames() const override;

};
#endif // TABLEMODEL_H
